package me.unurled.sacredrealms.sr.components.item;

import static me.unurled.sacredrealms.sr.utils.Component.comp;
import static me.unurled.sacredrealms.sr.utils.Items.lore;

import java.util.HashMap;
import java.util.List;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.item.abilities.Ability;
import me.unurled.sacredrealms.sr.components.item.enchantments.Enchantment;
import org.bukkit.Material;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

/* <b><color of the rarity>rarity
 * <dark_gray>Description: blablabla
 *   <dark_gray>blabbla
 *   <dark_gray>blabla
 * <gray>Attributes:
 *   <gray>➤ Health: <green>+100
 *   <gray>➤ Strength: <red>+10
 *   <gray>➤ Mana: <dark_aqua>+10
 * <gray>Abilities:
 *   <gray>➤ Ability 1,<yellow> On Rightclick unleash a powerful attack
 *               <dark_gray>cooldown: 10s,
 *               <dark_gray>damage: 100,
 *               <dark_gray>mana cost: <dark_aqua>50
 *   <gray>➤ Ability 2,<yellow> On Rightclick unleash a powerful attack
 *               <dark_gray>cooldown: 10s,
 *               <dark_gray>damage: 100,
 *               <dark_gray>mana cost: <dark_aqua>50
 * <dark_purple>Enchantments:
 *   <blue>➤ Enchantment <if max level gold>1
 *   <blue>➤ Enchantment <if max level gold>2
 * */
public class Item {

  @NotNull private final String ID;

  @NotNull private String name;

  @NotNull private Material material;

  @NotNull private String description;

  @NotNull private Integer customModelData;

  @NotNull private Rarity rarity;

  @NotNull private HashMap<Attribute, Double> attributes;

  @NotNull private HashMap<Enchantment, Integer> enchantments;

  @NotNull private List<Ability> abilities;

  public Item(@NotNull String ID, @NotNull String name, @NotNull Material material) {
    this.ID = ID;
    this.name = name;
    this.material = material;
    this.description = "";
    this.rarity = Rarity.COMMON;
    this.customModelData = 0;
    this.abilities = List.of();
    this.attributes = new HashMap<>();
    this.enchantments = new HashMap<>();
  }

  public @NotNull String getID() {
    return ID;
  }

  public @NotNull String getName() {
    return name;
  }

  public void setName(@NotNull String name) {
    this.name = name;
  }

  public Material getMaterial() {
    return material;
  }

  public void setMaterial(@NotNull Material material) {
    this.material = material;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Rarity getRarity() {
    return rarity;
  }

  public void setRarity(Rarity rarity) {
    this.rarity = rarity;
  }

  public HashMap<Attribute, Double> getAttributes() {
    return attributes;
  }

  public void setAttributes(HashMap<Attribute, Double> attributes) {
    this.attributes = attributes;
  }

  public Double getAttribute(Attribute attribute) {
    return attributes.get(attribute);
  }

  public void addAttribute(Attribute attribute, double value) {
    attributes.put(attribute, value);
  }

  public void removeAttribute(Attribute attribute) {
    attributes.remove(attribute);
  }

  public HashMap<Enchantment, Integer> getEnchantments() {
    return enchantments;
  }

  public void setEnchantments(HashMap<Enchantment, Integer> enchantments) {
    this.enchantments = enchantments;
  }

  public void addEnchantment(Enchantment enchantment, int level) {
    enchantments.put(enchantment, level);
  }

  public void removeEnchantment(Enchantment enchantment) {
    enchantments.remove(enchantment);
  }

  public List<Ability> getAbilities() {
    return abilities;
  }

  public void setAbilities(List<Ability> abilities) {
    this.abilities = abilities;
  }

  public void addAbility(Ability ability) {
    abilities.add(ability);
  }

  public void removeAbility(Ability ability) {
    abilities.remove(ability);
  }

  public Integer getCustomModelData() {
    return customModelData;
  }

  public void setCustomModelData(Integer customModelData) {
    this.customModelData = customModelData;
  }

  public ItemStack toItemStack() {
    ItemStack item = new ItemStack(material);
    ItemMeta meta = item.getItemMeta();
    if (meta == null) {
      return null;
    }

    PersistentDataContainer p = meta.getPersistentDataContainer();
    p.set(ItemManager.ID, PersistentDataType.STRING, ID);

    meta.displayName(comp(name).color(rarity.getColor()));

    if (customModelData != 0) {
      meta.setCustomModelData(customModelData);
    }

    lore(item);

    item.setItemMeta(meta);
    return item;
  }

  @Override
  public String toString() {
    return "{"
        + "ID: "
        + ID
        + ", name: "
        + name
        + ", material: "
        + material
        + ", description: "
        + description
        + ", customModelData: "
        + customModelData
        + ", rarity: "
        + rarity
        + ", attributes: "
        + attributes
        + ", enchantments: "
        + enchantments
        + ", abilities: "
        + abilities
        + "}";
  }
}
